@@ -18,6 +18,8 @@
   #:use-module (oop goops)
   #:use-module (srfi srfi-1)
   #:export (aa-delete
+            aa-for-each
+            aa-for-each-reverse
             aa-insert
             aa-map
             aa-map-keys
@@ -256,18 +258,37 @@ not found, the original unmodified tree will be returned."
                                 (descend-right (right-tree t))))))
     (descend-right (left-tree tree))))
 
-(define (aa-map proc tree)
-"Maps proc over the key-value pairs in tree, moving from smallest to
-greatest key, returning the list of results. proc must take an
-argument of the form (key . value). Does not return results."
-  (if (left-tree tree) (aa-map proc (left-tree tree)))
+(define (aa-for-each proc tree)
+"Calls proc for each of the key-value pairs in tree, moving from
+smallest to greatest key. proc must take an argument of the form (key
+. value). Does not return results."
+  (if (left-tree tree) (aa-for-each proc (left-tree tree)))
   (proc (key-value tree))
-  (if (right-tree tree) (aa-map proc (right-tree tree))))
+  (if (right-tree tree) (aa-for-each proc (right-tree tree))))
+
+(define (aa-for-each-reverse proc tree)
+"Calls proc for each of the key-value pairs in tree, moving from
+smallest to greatest key. proc must take an argument of the form (key
+. value). Does not return results."
+  (if (right-tree tree) (aa-for-each-reverse proc (right-tree tree)))
+  (proc (key-value tree))
+  (if (left-tree tree) (aa-for-each-reverse proc (left-tree tree))))
+
+(define (aa-map fn tree)
+  (let ((lst '()))
+    (aa-for-each-reverse
+     (lambda (p)
+       (set! lst (cons (fn p) lst)))
+     tree)
+    lst))
 
 (define (aa-map-reverse fn tree)
-  (if (right-tree tree) (aa-map fn (right-tree tree)))
-  (fn (key-value tree))
-  (if (left-tree tree) (aa-map fn (left-tree tree))))
+  (let ((lst '()))
+    (aa-for-each
+     (lambda (p)
+       (set! lst (cons (fn p) lst)))
+     tree)
+    lst))
 
 (define (aa-map-keys fn tree)
   (aa-map (lambda (kv) (fn (car kv))) tree))
@@ -276,12 +297,9 @@ argument of the form (key . value). Does not return results."
   (aa-map (lambda (kv) (fn (cdr kv))) tree))
 
 (define (aa-to-list tree)
-"Return the key-value pairs of the tree in a list. Pairs are order by
+  "Return the key-value pairs of the tree in a list. Pairs are order by
 key from smallest to greatest."
-  (let ((l '()))
-    (aa-map-reverse
-     (lambda (kv)
-       (set! l (cons kv l))) tree) l))
+  (aa-map identity tree))
 
 ;; Basic idea here is that
 ;; aa-insert tree key1 key2 value =
